@@ -25,7 +25,7 @@ except ImportError:
     import z0lib
 
 
-__version__ = "2.0.14"
+__version__ = "2.0.15"
 python_version = "%s.%s" % (sys.version_info[0], sys.version_info[1])
 
 #
@@ -35,10 +35,16 @@ python_version = "%s.%s" % (sys.version_info[0], sys.version_info[1])
 REQVERSION = {
     "acme_tiny": {"6.1": ">=4.0.3"},
     "argparse": {"0": "==1.2.1"},
-    # "astroid": {"2.7": "==1.6.5", "3.5": "==2.2.0"},  # Version by test pkgs
-    "astroid": {"2.7": "==1.6.5", "3.5": "==2.11.7"},  # Version by test pkgs
+    "astroid": {
+        # Version by test pkgs
+        "2.7": "==1.6.5", "3.5": "==2.11.7", "3.7": ">=2.11.7,<=3.1.0"},
     "autopep8": {"0": "==1.2"},
-    "Babel": {"6.1": "==1.3", "8.0": "==2.3.4", "16.0": ">=2.6.0,<=2.9.1"},
+    "Babel": {
+        "6.1": "==1.3",
+        "8.0": "==2.3.4",
+        "11.0": ">=2.3.4,<=2.9.1",
+        "16.0": ">=2.6.0,<=2.9.1"
+    },
     "beautifulsoup": {"6.1": "==3.2.1"},
     "bokeh": {
         "10.0": "bokeh<1.4.0",
@@ -49,18 +55,26 @@ REQVERSION = {
     },
     "codicefiscale": {"6.1": "==0.9"},
     "coverage": {"2.7": "<5.6.0", "3.5": ">=5.0.0"},
-    "cryptography": {"2.7": ">=2.2.2,<3.4", "3.7": ">=23.0,<38.0", "3.9": ">38.0.2"},
-    "decorator": {"6.1": "==3.4.0", "10.0": "==4.0.10"},
+    "cryptography": {
+        "15.0": ">=2.6.1,<38.0",
+        "16.0": "PYVER",
+        "2.7": ">=2.2.2,<3.4",
+        "3.7": ">=2.6.1,<38.0",
+        "3.9": ">38.0.2"
+    },
+    "decorator": {"6.1": "==3.4.0", "10.0": "==4.0.10", "11.0": ">=4.0.10"},
     # "docutils": {"0": "==0.14", "6.1": "==0.12", "3.7": "==0.16"},
     "docutils": {"0": "==0.16"},       # By test pkgs
     "ebaysdk": {"6.1": "==2.1.4"},
     "email_validator": {"10.0": "<1.3.0", "12.0": ">=1.3"},
     "ERPpeek": {"0": "==1.6.1"},
+    "evdev": {"2.7": "<1.5", "3.7": ">=1.0"},
     "factur-x": {"2.7": "<3.0", "3.6": ">=3.0"},
     "feedparser": {"6.1": "==5.1.3", "10.0": "==5.2.1"},
     "flake8": {
         "2.7": "<4.0.0",     # <<Tested 3.5.0; 3.6.0 does not work>>
         "3.6": ">=4.0.0",
+        "3.7": ">=5.0.0",
     },
     "gdata": {"6.1": "==2.0.18"},
     "gevent": {
@@ -71,19 +85,33 @@ REQVERSION = {
         "12.0": ">=1.5.0,<=21.12.0",     # https://github.com/gevent/gevent/issues/1899
         "14.0": ">=20.9.0,<=22.10.99",   # by odoo + gevent documentation
         "15.0": "PYVER",
-        "3.7": ">=22.10.0,<=22.10.99",
+        "3.7": ">=1.5.0,<=22.10.99",
+        "3.8": ">=20.9.0,<=22.10.99",
+        "3.10": ">=21.8.0,<=22.10.99",
     },
     "greenlet": {
         "6.1": "==0.4.2",
         "7.0": ">=0.4.7,<=0.4.10",       # by odoo documentation + test
         "3.7": ">=0.4.13",
     },
+    "holidays": {
+        "2.7": "<0.10",
+        "3.7": ">=0.10",
+    },
     "importlib-metadata": {"3.7": "==4.8.3"},
     "invoice2data": {"2.7": "<0.4", "3.6": ">=0.4"},
     "ipy": {"6.1": ">=0.83"},
-    "isort": {"0": "==4.3.4"},  # Version by test pkgs
+    "isort": {"0": "==4.3.4", "3.10": ">=4.3.4"},  # Version by test pkgs
     "jcconv": {"6.1": "==0.2.3"},
-    "Jinja2": {"6.1": "==2.7.3", "9.0": "==2.8.1", "10.0": "==2.10.1"},
+    "Jinja2": {
+        "6.1": "==2.7.3",
+        "9.0": "==2.8.1",
+        "10.0": "==2.10.1",
+        "12.0": "PYVER",
+        "3.6": "==2.10.1",
+        "3.8": ">=2.11.2,<=2.11.3",
+        "3.10": "==3.1.2",
+    },
     "jupyter-server": {"0": "<1.20.0"},
     "lessc": {"0": ">=3.0.0"},
     "lxml": {
@@ -92,7 +120,7 @@ REQVERSION = {
         "9.0": ">=3.4.1,<=3.4.4",
         "10.0": ">=3.5.0,<=3.6.4",
         "11.0": "PYVER",
-        "14.0": ">=4.5.0",
+        # "14.0": ">=4.5.0",
         "15.0": ">=4.6.5",
         "16.0": "PYVER",
         "3.6": "==3.7.1",
@@ -100,26 +128,39 @@ REQVERSION = {
         "3.8": ">=4.6.1,<=4.6.5",
         "3.10": "==4.9.2",
     },
-    "mccabe": {"0": "<0.7.0,>=0.6.0", "3.6": "<0.8.0,>=0.7.0"},
+    "mccabe": {"0": "<0.7.0,>=0.6.0", "3.8": "<0.8.0,>=0.7.0"},
     "Mako": {
         "6.1": "==1.0.0", "7.0": "==1.0.1", "8.0": "==1.0.4", "10.0": ">=1.0.4",
     },
-    "MarkupSafe": {"6.1": "==0.23", "14.0": "==1.1.0", "0": "<2.1.0"},  # Tested 1.0
+    "MarkupSafe": {
+        "0": "<2.1.0",
+        "6.1": "==0.23",
+        "14.0": "==1.1.0",
+        "15.0": "PYVER",
+        "3.8":  ">=1.1.0,<=1.1.1",
+        "3.10":  "==2.1.2",
+        "3.12":  "==2.1.5",
+    },  # Tested 1.0
     "matplotlib": {
         "10.0": "==3.0.3",
         "11.0": "PYVER",
         # "13.0": "==3.4.1",
         "3.6": "==3.0.3",
-        # "3.7": "==3.4.1"
         "3.7": "<3.3.0"     # Experimental!
     },
     "mock": {"6.1": "==1.0.1", "8.0": "==2.0.0"},
     "nbconvert": {"0": "==6.0.7"},
     "num2words": {"2.7": "<=0.5", "3.6": ">=0.5"},
     "odoo_score": {"6.1": "==1.0.1", "10.0": ">=2.0.0"},
+    "oerplib3": {
+        "0": ">=0.8.4", "12.0": "PYVER", "3.8": ">=0.8.4", "3.10": ">=0.8.5"
+    },
     "ofxparse": {"6.1": "==0.16"},
-    "pandas": {"3.7": ">=0.22.0,<=1.1.0"},
-    "passlib": {"6.1": "==1.6.2", "10.0": "==1.6.5", "16.0": ">=1.7.0"},
+    "pandas": {"0": ">=0.22.0,<=1.1.0", "3.7": ">=0.22.0,<=1.1.0", "3.8": ">=0.25.3"},
+    "passlib": {
+        "6.1": "==1.6.2", "10.0": "==1.6.5", "11.0": ">=1.6.5", "16.0": ">=1.7.0"
+    },
+    "pdfplumber": {"14.0": "<0.10.2"},
     "Pillow": {
         "6.1": "==3.4.1",
         "7.0": "==3.4.2",
@@ -131,7 +172,24 @@ REQVERSION = {
         "3.7": ">=6.1.0,<=6.2.0",
         "3.8": ">=6.2.1",
     },
-    "psutil": {"6.1": "==2.1.1", "7.0": "==2.2.0", "8.0": "==4.3.1"},
+    "postgresql": {
+        "6.1": ">=8.0,<=10.0",
+        "7.0": ">=9.5,<=12.0",
+        "8.0": ">=9.6,<=12.0",
+        "9.0": ">=10.0,<=12.0",
+        "10.0": ">=10.0,<=12.0",
+        "11.0": "==12.0",
+        "12.0": "==12.0",
+        "13.0": ">=12.0,<=13.0",
+        "14.0": ">=12.0,<=14.0",
+        "15.0": ">=12.0,<=15.0",
+        "16.0": ">=14.0,<=15.0",
+        "17.0": "==16.0",
+        "18.0": "==16.0",
+    },
+    "psutil": {
+        "6.1": "==2.1.1", "7.0": "==2.2.0", "8.0": "==4.3.1", "11.0": ">=4.3.1"
+    },
     "psycogreen": {"6.1": "==1.0"},
     "psycopg2-binary": {
         "6.1": ">=2.0.0,<2.7.3.3",
@@ -141,7 +199,7 @@ REQVERSION = {
         "0": ">=2.7.4",
     },
     # "pycodestyle": {"0": "==2.3.1"},
-    "pydot": {"6.1": "==1.0.2", "8.0": "==1.2.3"},
+    "pydot": {"6.1": "==1.0.2", "8.0": "==1.2.3", "11.0": ">=1.2.3"},
     # "pyflakes": {"0": "pyflakes<1.6.0,>=1.5.0"},
     "Pygments": {
         "10.0": "==2.2.0",
@@ -165,7 +223,7 @@ REQVERSION = {
         # "16.0": ">=7.0.0",      # from pypi documentation
         "2.7": "<=4.0.0",
         "3.5": "<=5.0.0",
-        # "3.8": ">=3.5.0,<=5.0.0",
+        "3.7": ">5.0.0,<=8.0.0",
         "3.8": ">=8.0",
     },
     "pylint-plugin-utils": {
@@ -175,38 +233,49 @@ REQVERSION = {
     },
     "pyOpenSSL": {
         "0": ">=16.2.0,<19.0",
-        "3.8": ">=20.0.0,<23.0",
-        "3.9": ">=22.0.0,<23.0"
+        "3.8": ">=19.0.0,<23.0",
+        "3.9": ">=21.0.0,<23.0"
     },
     "pyotp": {"2.7": "==2.3.0", "3.5": ">=2.4.0"},
-    "pyPDF2": {"2.7": "==1.28.4", "3.5": "<2.0"},
+    "pyPDF2": {"2.7": "==1.28.4", "3.5": "<2.0", "3.8": "<3.0"},
     "pysftp": {"6.1": ">=0.2.9"},
     "pyparsing": {"6.1": "==1.5.7", "7.0": "==2.0.3", "10.0": "==2.1.10"},
-    "pyPdf": {"6.1": "==1.13"},
+    "pyPdf": {"6.1": "==1.13", "12.0": ">=3.1.0"},
     "pyserial": {"6.1": "==2.7", "10.0": ">=3.1.1"},
     "Python-Chart": {"6.1": "==1.39"},
     "python-dateutil": {
-        "6.1": "==1.5", "7.0": "==2.4.0", "8.0": "==2.5.3", "11.0": "==2.5.3"
+        "6.1": "==1.5",
+        "7.0": "==2.4.0",
+        "8.0": "==2.5.3",
+        "11.0": "==2.5.3",
+        "14.0": ">=2.5.3",
     },
     "python-ldap": {
         "6.1": "==2.4.15",
         "7.0": "==2.4.19",
         "10.0": "==2.4.27",
-        "11.0": "==2.5.28",
+        "11.0": ">=3.0.0,<3.1.0",
         "13.0": "==3.1.0",
+        "15.0": ">=3.1.0,<=3.4.4",
     },
     "python-openid": {"6.1": "==2.2.5"},
     "python-stdnum": {"6.1": ">=1.8.1"},
     "pytz": {"6.1": ">=2014.10", "10.0": ">=2016.7"},
     # check for: Odoo requires >=1.0.0b1, but it does conflict with other packges
-    "pyusb": {"6.1": ">=1.0.0b1", "10.0": "==1.0.0", "16.0": ">=1.0.0b1"},
+    "pyusb": {"6.1": ">=1.0.0b1", "10.0": "==1.0.0", "14.0": ">=1.0.0b1"},
     "pyxb": {"6.1": "==1.2.4", "8.0": "==1.2.5", "10.0": ">=1.2.5", "12.0": "==1.2.6"},
     "PyWebDAV": {"6.1": "<0.9.8"},
     "PyYAML": {"6.1": "==3.11", "8.0": "==3.12", "3.7": "==3.13", "3.9": "==6.0"},
-    "qrcode": {"6.1": "==5.0.1", "7.0": "==5.1", "10.0": "==5.3"},
+    "qrcode": {
+        "6.1": "==5.0.1", "7.0": "==5.1", "10.0": "==5.3", "15.0": ">=5.3"
+    },
     "readme-renderer" : {"2.7": "<25.0", "3.5": "<29.0", "3.6": ">=30.0"},
-    "restructuredtext_lint": {"6.1": "==0.12.2", "0": "==1.1.3"},
-    "reportlab": {"6.1": "==3.1.44", "10.0": "==3.3.0"},
+    "restructuredtext_lint": {
+        "6.1": "==0.12.2", "15.0": ">=1.0", "0": "==1.1.3",
+    },
+    "reportlab": {
+        "6.1": "==3.1.44", "10.0": "==3.3.0", "14.0": ">=3.3.0",
+    },
     "requests": {
         "6.1": "==2.6.0",
         "10.0": ">=2.11.1,<=2.20.0",
@@ -224,7 +293,7 @@ REQVERSION = {
     "Sphinx": {"2.7": "==1.2.3", "3.7": ">=2.4.0"},
     "suds": {"6.1": "==0.4"},
     "suds-jurko": {"6.1": "==0.6"},
-    "translators": {"0": "<5.0.0"},
+    "translators": {"0": "<4.11.3"},
     "unicodecsv": {"6.1": ">=0.14.1"},
     "unidecode": {"6.1": "==0.4.17", "10.0": "<=1.2.0", "11.0": ">1.2.0"},
     "unittest2": {"6.1": "==0.5.1", "11.0": ">=1.0.0"},
@@ -240,7 +309,6 @@ REQVERSION = {
         "14.0": "==0.16.1",
         "15.0": "PYVER",
         "3.9": "==2.0.2",
-
     },
     "wkhtmltopdf": {"6.1": "==0.12.1", "10.0": "==0.12.4", "12.0": "==0.12.5"},
     "wok_code": {"6.1": "<2.0.6", "10.0": ">=2.0.0"},
@@ -249,7 +317,7 @@ REQVERSION = {
     "xlrd": {"6.1": "==1.0.0"},
     "xlwt": {"6.1": "==0.7.5", "10.0": "==1.1.2", "12.0": "==1.3"},
     "z0bug_odoo": {"6.1": "==1.0.1", "10.0": ">=2.0.0"},
-    "zeep": {"0": "==2.4.0"},
+    "zeep": {"0": "==2.4.0", "17.0": ">=2.4.0,<5.0"},
     "zerobug": {"6.1": "==1.0.1", "10.0": ">=2.0.0"},
 }
 ALIAS = {
@@ -336,12 +404,20 @@ FORCE_ALIAS3 = {
     "pytz==2016.7": "pytz>=2016.7",
     "pytz==2019.3": "pytz>=2019.3",
     "zeep==4.0.0": "zeep==2.4.0",
-    "python-ldap": "python-ldap==3.1.0",
+    # "python-ldap": "python-ldap==3.1.0",
     "Mako==1.0.4": "Mako>=1.0.4",
     "rich": "rich<=12.0.0",
     "importlib-metadata": "importlib-metadata==4.8.3",
     "pygount": "pygount<=1.2.0",
-    "pandas": "pandas>=0.22.0,<=1.1.0",
+    # "pandas": "pandas>=0.22.0,<=1.1.0",
+}
+FORCE_ALIAS_OS = {
+    "wkhtmltopdf==0.12.4": "wkhtmltopdf==0.12.6",
+    "wkhtmltopdf==0.12.5": "wkhtmltopdf==0.12.6",
+}
+FORCE_ALIAS_OS = {
+    "wkhtmltopdf==0.12.4": "wkhtmltopdf==0.12.6",
+    "wkhtmltopdf==0.12.5": "wkhtmltopdf==0.12.6",
 }
 PIP_SECURE_PACKAGES = [
     "urllib3[secure]",
@@ -412,6 +488,7 @@ PIP3_TEST_PACKAGES = [
     "lazy_object_proxy",
     "lxml",
     "MarkupSafe",
+    "mccabe",
     "mock",
     "pbr",
     "polib",
@@ -451,7 +528,7 @@ BIN_TEST_PACKAGES = [
     "python-setuptools",
 ]
 RPC2_PACKAGES = ["clodoo", "odoorpc", "oerplib"]
-RPC3_PACKAGES = ["clodoo", "odoorpc"]
+RPC3_PACKAGES = ["clodoo", "odoorpc", "oerplib3"]
 PIP_BASE_PACKAGES = [
     "Babel",
     "chardet",
@@ -500,9 +577,12 @@ PIP_ODOO_BASE_PACKAGES = [
     "lessc; odoo_major>=10",
     "lxml",
     "Mako",
+    "matplotlib; odoo_major>=14",
     "numpy",
     "num2words",
+    "odoo_score",
     "passlib",
+    "pdfplumber; odoo_major>=14",
     "Pillow",
     "psutil",
     "psycogreen",
@@ -601,14 +681,19 @@ def get_naked_pkgname(pkg):
     return re.split('[!<=>@#;]', python_plus.qsplit(pkg)[0])[0].strip()
 
 
+def get_odoo_majver(odoo_ver):
+    return int(odoo_ver.split(".")[0]) if odoo_ver else 0
+
+
 def eval_requirement_cond(line, pyver=None, odoo_ver=None):
-    pyver = pyver or '3.7'
     items = line.split('#')[0].split(";")
     if len(items) == 1:
         return get_naked_pkgname(line)
-    testenv = {"sys_platform": sys.platform,
-               "python_version": pyver,
-               "odoo_major": int(odoo_ver.split(".")[0])}
+    testenv = {
+        "sys_platform": sys.platform,
+        "python_version": pyver,
+        "odoo_major": get_odoo_majver(odoo_ver),
+    }
     if eval(items[1], testenv):
         return get_naked_pkgname(line)
     return False
@@ -695,7 +780,7 @@ def name_n_version(full_item, with_version=None, odoo_ver=None, pyver=None):
     if "openupgradelib" not in item_l and item_l in ALIAS:
         full_item = full_item.replace(item, ALIAS[item_l])
         item = ALIAS[item_l]
-    if odoo_ver and int(odoo_ver.split('.')[0]) > 10:
+    if get_odoo_majver(odoo_ver) > 10:
         if "openupgradelib" not in item and item in ALIAS3:
             full_item = full_item.replace(item, ALIAS3[item])
             item = ALIAS3[item]
@@ -768,7 +853,9 @@ def name_n_version(full_item, with_version=None, odoo_ver=None, pyver=None):
         else:
             full_item = FORCE_ALIAS2.get(full_item, full_item)
     else:
-        if full_item in FORCE_ALIAS3:
+        if full_item in FORCE_ALIAS_OS:
+            full_item = FORCE_ALIAS_OS[full_item]
+        elif full_item in FORCE_ALIAS3:
             full_item = FORCE_ALIAS3[full_item]
         else:
             full_item = FORCE_ALIAS3.get(item, full_item)
@@ -938,7 +1025,7 @@ def add_package(deps_list, kw, item, with_version=None, odoo_ver=None, pyver=Non
         if with_version and full_item:
             found = False
             for ix, kk in enumerate(deps_list[kw2]):
-                if kk.startswith(item):
+                if get_naked_pkgname(kk) == item:
                     deps_list[kw2][ix] = merge_item_version(kk, full_item)
                     found = True
                     break
@@ -1152,7 +1239,7 @@ def walk_dir(cdir, manifests, reqfiles, setups, read_from_manifest, recurse):
 
 
 def get_pyver_4_odoo(odoo_ver):
-    odoo_major = int(odoo_ver.split(".")[0])
+    odoo_major = get_odoo_majver(odoo_ver)
     if odoo_major <= 10:
         pyver = "2.7"
     else:
@@ -1347,11 +1434,13 @@ def main(cli_args=None):
     parser.add_argument("-v")
     parser.add_argument("-y", "--python", action="store", dest="pyver")
     ctx = parser.parseoptargs(sys.argv[1:], apply_conf=False)
+    if ctx["odoo_ver"] and not ctx["pyver"]:
+        ctx = get_pyver(ctx)
+    if not ctx["pyver"]:
+        ctx["pyver"] = "3.10"
     if ctx["pyver"]:
         global PY3_DEV
         PY3_DEV = "python%s-dev" % ctx["pyver"]
-    if ctx["odoo_ver"] and not ctx["pyver"]:
-        ctx = get_pyver(ctx)
     if ctx["out_file"]:
         ctx = set_def_outfile(ctx)
     if (
@@ -1539,7 +1628,7 @@ def main(cli_args=None):
             pyver=ctx["pyver"],
         )
     if ctx["rpc_packages"]:
-        if ctx["odoo_ver"] and int(ctx["odoo_ver"].split(".")[0]) <= 8:
+        if ctx["odoo_ver"] and int(ctx["odoo_ver"].split(".")[0]) <= 10:
             deps_list = package_from_list(
                 deps_list,
                 "python",
@@ -1571,13 +1660,13 @@ def main(cli_args=None):
                      + RPC3_PACKAGES):
             rm_list = []
             for ii, dep_pkg in enumerate(deps_list["python"]):
-                if deps_list["python"][ii].startswith(item):
+                if get_naked_pkgname(deps_list["python"][ii]) == item:
                     rm_list.append(ii)
             while len(rm_list):
                 ii = rm_list.pop()
                 del deps_list["python"][ii]
             for ii, dep_pkg in enumerate(deps_list["bin"]):
-                if deps_list["bin"][ii].startswith(item):
+                if get_naked_pkgname(deps_list["bin"][ii]) == item:
                     rm_list.append(ii)
             while len(rm_list):
                 ii = rm_list.pop()
@@ -1652,8 +1741,3 @@ def main(cli_args=None):
 
 if __name__ == "__main__":
     exit(main())
-
-
-
-
-
